%Copyright (c) 2019, Md Ashikuzzaman and Hassan Rivaz
%All rights reserved.

%Redistribution and use in source and binary forms, with or without
%modification, are permitted provided that the following conditions are met:

% Redistributions of source code must retain the above copyright notice, this
% list of conditions and the following disclaimer.

% Redistributions in binary form must reproduce the above copyright notice,
% this list of conditions and the following disclaimer in the documentation
% and/or other materials provided with the distribution.

% Neither the name of the copyright holder nor the names of its
%  contributors may be used to endorse or promote products derived from
%  this software without specific prior written permission.

%THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
%AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
%IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
%DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
%FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
%DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
%SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
%CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
%OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
%OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


% Please cite the following papers if you are using this code 

% [1] Ashikuzzaman, M., Belasso, C., Kibria, M. G., Bergdahl, A., Gauthier, 
%C. J., Rivaz, H., "Low Rank and Sparse Decomposition of Ultrasound Color Flow
%Images for Suppressing Clutter in Real-Time", IEEE Transactions on Medical Imaging, 2019 (in press).

% [2] Rezaei, B. and Ostadabbas, S., "Background Subtraction via Fast Robust 
%Matrix Completion", ICCV 2017.



% Please cite the following paper if you are using the accompanying RF
% data

% [1] Ashikuzzaman, M., Belasso, C., Kibria, M. G., Bergdahl, A., Gauthier, 
%C. J., Rivaz, H., "Low Rank and Sparse Decomposition of Ultrasound Color Flow
%Images for Suppressing Clutter in Real-Time", IEEE Transactions on Medical Imaging, 2019 (in press).

%For downloading the dataset associated with this paper, please visit 
%IMPACT Database (https://users.encs.concordia.ca/~impact/database/)


%------------------------------------------------------------------%
clc
close all;
clear variables;

%% Specifying the number of frames and the maximum number of iterations
max_iter=100; %Maximum number of iterations
fr_num=20; %Number of frames used to form the Casorati Matrix

%% Specifying the data folder

folder='C:\...\data_RAPID\data_flow_phantom_plane_wave_water_with_bubble'; %use full path to your data folder
f=dir(fullfile(folder, '*.mat'));

%% Reading the data and creating the Casorati Matrix
X=[]; %Casorati Matrix
for i=1:fr_num
    name=fullfile(folder, f(i).name);
    filename=sprintf('%d_layer0_idx%d_BDATA_BF',i);
    load(name,'BfData'); %loading RF data
    Im1=BfData(401:1400,41:end); %Selecting the intended portion of the image
    env1=hilbert(Im1(:,:)/max(Im1(:))); %Envelope of RF data
    env1=env1(1:4:end,:); %Downsampling of envelope
    [height width]=size(env1);
    env1=reshape(env1,height*width,1); %Converting frames to vectors 
    X=[X env1]; %Stacking the vectors as columns of X 
end
X=double(X);

%% Applying RAPID on the Casorati matrix X 

%Solving the rank-minimizing problem by in-face Extended Frank-Wolfe method
[L, ~ ]=InfaceExtFrankWolfe(X,[],[],max_iter);

%Obtaining the sparse component
S=abs(X-L);

Pow_E=sum(S.^2,2)/fr_num; %Obtaining the power average of the sparse frames
Clut_sup=reshape(Pow_E,height,width); %Back to the original shape

%% Showing the clutter suppressed image
figure;imagesc([0 3],[0.77 2.7],Clut_sup(:,1:114),[0 .007]); 
colormap(hot);colorbar;
xlabel('width (cm)');ylabel('depth (cm)');
ax=gca; % current axes
ax.FontSize = 16;
